@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -24,7 +24,6 @@
 
 #include <glib-object.h>
 
-#include "gdm-display.h"
 #include "gdm-manager-glue.h"
 
 G_BEGIN_DECLS
@@ -40,18 +39,18 @@ typedef struct GdmManagerPrivate GdmManagerPrivate;
 
 typedef struct
 {
-        GdmDBusManagerSkeleton  parent;
+        GDBusInterfaceSkeleton  parent;
         GdmManagerPrivate      *priv;
 } GdmManager;
 
 typedef struct
 {
-        GdmDBusManagerSkeletonClass parent_class;
+        GDBusInterfaceSkeletonClass parent_class;
 
         void          (* display_added)    (GdmManager      *manager,
                                             const char      *id);
         void          (* display_removed)  (GdmManager      *manager,
-                                            GdmDisplay      *display);
+                                            const char      *id);
 } GdmManagerClass;
 
 typedef enum
@@ -67,11 +66,11 @@ GType               gdm_manager_get_type                       (void);
 GdmManager *        gdm_manager_new                            (void);
 void                gdm_manager_start                          (GdmManager *manager);
 void                gdm_manager_stop                           (GdmManager *manager);
+void                gdm_manager_set_wait_for_go                (GdmManager *manager,
+                                                                gboolean    wait_for_go);
 
 void                gdm_manager_set_xdmcp_enabled              (GdmManager *manager,
                                                                 gboolean    enabled);
-void                gdm_manager_set_show_local_greeter         (GdmManager *manager,
-                                                                gboolean    show_local_greeter);
 gboolean            gdm_manager_get_displays                   (GdmManager *manager,
                                                                 GPtrArray **displays,
                                                                 GError    **error);
