@@ -40,6 +40,8 @@
 
 #include <X11/Xlib.h> /* for Display */
 
+#include <act/act-user-manager.h>
+
 #include "gdm-common.h"
 
 #include "gdm-settings-client.h"
@@ -50,7 +52,7 @@
 #include "gdm-server.h"
 #include "gdm-session.h"
 #include "gdm-session-glue.h"
-#include "gdm-welcome-session.h"
+#include "gdm-launch-environment.h"
 #include "gdm-settings-direct.h"
 #include "gdm-settings-keys.h"
 
@@ -62,6 +64,8 @@
 #define MAX_CONNECT_ATTEMPTS  10
 #define DEFAULT_PING_INTERVAL 15
 
+#define INITIAL_SETUP_USERNAME "gdm-initial-setup"
+
 struct GdmSimpleSlavePrivate
 {
         GPid               pid;
@@ -76,8 +80,12 @@ struct GdmSimpleSlavePrivate
         guint              connection_attempts;
 
         GdmServer         *server;
+
+        /* we control the user session */
         GdmSession        *session;
-        GdmWelcomeSession *greeter;
+
+        /* this spawns and controls the greeter session */
+        GdmLaunchEnvironment *greeter_environment;
 
         GHashTable        *open_reauthentication_requests;
 
@@ -103,7 +111,6 @@ static void     gdm_simple_slave_finalize       (GObject             *object);
 G_DEFINE_TYPE (GdmSimpleSlave, gdm_simple_slave, GDM_TYPE_SLAVE)
 
 static void create_new_session (GdmSimpleSlave  *slave);
-static void start_greeter      (GdmSimpleSlave *slave);
 static void start_session      (GdmSimpleSlave  *slave);
 static void queue_start_session (GdmSimpleSlave *slave,
                                  const char     *service_name);
@@ -324,7 +331,7 @@ stop_greeter (GdmSimpleSlave *slave)
 
         g_debug ("GdmSimpleSlave: Stopping greeter");
 
-        if (slave->priv->greeter == NULL) {
+        if (slave->priv->greeter_environment == NULL) {
                 g_debug ("GdmSimpleSlave: No greeter running");
                 return;
         }
@@ -340,7 +347,7 @@ stop_greeter (GdmSimpleSlave *slave)
         }
         g_free (username);
 
-        gdm_welcome_session_stop (GDM_WELCOME_SESSION (slave->priv->greeter));
+        gdm_launch_environment_stop (GDM_LAUNCH_ENVIRONMENT (slave->priv->greeter_environment));
 }
 
 static void
@@ -388,11 +395,11 @@ start_session_timeout (GdmSimpleSlave  *slave)
                 g_free (slave->priv->start_session_service_name);
                 slave->priv->start_session_service_name = NULL;
         } else {
-                if (slave->priv->greeter == NULL) {
+                if (slave->priv->greeter_environment == NULL) {
                         /* auto login */
                         start_session (slave);
                 } else {
-                        /* Session actually gets started from on_greeter_session_stop */
+                        /* Session actually gets started from on_greeter_environment_session_stop */
                         stop_greeter (slave);
                 }
         }
@@ -457,9 +464,7 @@ on_session_conversation_started (GdmSession       *session,
                 return;
         }
 
-        if (delay > 0) {
-                gdm_session_request_timed_login (session, username, delay);
-        } else {
+        if (delay == 0) {
                 g_debug ("GdmSimpleSlave: begin auto login for user '%s'", username);
                 /* service_name will be "gdm-autologin"
                  */
@@ -540,12 +545,59 @@ on_session_client_ready_for_session_to_start (GdmSession      *session,
                 gdm_simple_slave_start_session_when_ready (slave, service_name);
         }
 }
+
+static void
+on_ready_to_request_timed_login (GdmSession         *session,
+                                 GSimpleAsyncResult *result,
+                                 gpointer           *user_data)
+{
+        int delay = GPOINTER_TO_INT (user_data);
+        GCancellable *cancellable;
+        char         *username;
+
+        cancellable = g_object_get_data (G_OBJECT (result),
+                                         "cancellable");
+        if (g_cancellable_is_cancelled (cancellable)) {
+                return;
+        }
+
+        username = g_simple_async_result_get_source_tag (result);
+
+        gdm_session_request_timed_login (session, username, delay);
+
+        g_object_weak_unref (G_OBJECT (session),
+                             (GWeakNotify)
+                             g_cancellable_cancel,
+                             cancellable);
+        g_object_weak_unref (G_OBJECT (session),
+                             (GWeakNotify)
+                             g_object_unref,
+                             cancellable);
+        g_object_weak_unref (G_OBJECT (session),
+                             (GWeakNotify)
+                             g_free,
+                             username);
+
+        g_free (username);
+}
+
+static gboolean
+on_wait_for_greeter_timeout (GSimpleAsyncResult *result)
+{
+        g_simple_async_result_complete (result);
+
+        return FALSE;
+}
+
 static void
 on_session_client_connected (GdmSession          *session,
                              GCredentials        *credentials,
                              GPid                 pid_of_client,
                              GdmSimpleSlave      *slave)
 {
+        gboolean timed_login_enabled;
+        char    *username;
+        int      delay;
         gboolean display_is_local;
 
         g_debug ("GdmSimpleSlave: client connected");
@@ -558,6 +610,64 @@ on_session_client_connected (GdmSession          *session,
         if ( ! display_is_local) {
                 alarm (0);
         }
+
+        timed_login_enabled = FALSE;
+        gdm_slave_get_timed_login_details (GDM_SLAVE (slave), &timed_login_enabled, &username, &delay);
+
+        if (! timed_login_enabled) {
+                return;
+        }
+
+        /* temporary hack to fix timed login
+         * http://bugzilla.gnome.org/680348
+         */
+        if (delay > 0) {
+                GSimpleAsyncResult *result;
+                GCancellable       *cancellable;
+                guint               timeout_id;
+                gpointer            source_tag;
+
+                delay = MAX (delay, 4);
+
+                cancellable = g_cancellable_new ();
+                source_tag = g_strdup (username);
+                result = g_simple_async_result_new (G_OBJECT (session),
+                                                    (GAsyncReadyCallback)
+                                                    on_ready_to_request_timed_login,
+                                                    GINT_TO_POINTER (delay),
+                                                    source_tag);
+                g_simple_async_result_set_check_cancellable (result, cancellable);
+                g_object_set_data (G_OBJECT (result),
+                                   "cancellable",
+                                   cancellable);
+
+                timeout_id = g_timeout_add_seconds_full (delay - 2,
+                                                         G_PRIORITY_DEFAULT,
+                                                         (GSourceFunc)
+                                                         on_wait_for_greeter_timeout,
+                                                         g_object_ref (result),
+                                                         (GDestroyNotify)
+                                                         g_object_unref);
+                g_cancellable_connect (cancellable,
+                                       G_CALLBACK (g_source_remove),
+                                       GINT_TO_POINTER (timeout_id),
+                                       NULL);
+
+                g_object_weak_ref (G_OBJECT (session),
+                                   (GWeakNotify)
+                                   g_cancellable_cancel,
+                                   cancellable);
+                g_object_weak_ref (G_OBJECT (session),
+                                   (GWeakNotify)
+                                   g_object_unref,
+                                   cancellable);
+                g_object_weak_ref (G_OBJECT (session),
+                                   (GWeakNotify)
+                                   g_free,
+                                   source_tag);
+        }
+
+        g_free (username);
 }
 
 static void
@@ -574,7 +684,7 @@ on_session_client_disconnected (GdmSession          *session,
                       "display-is-local", &display_is_local,
                       NULL);
 
-        if ( ! display_is_local) {
+        if ( ! display_is_local && !slave->priv->session_is_running) {
                 gdm_slave_stopped (GDM_SLAVE (slave));
         }
 }
@@ -602,8 +712,8 @@ create_new_session (GdmSimpleSlave  *slave)
 
         g_debug ("GdmSimpleSlave: Creating new session");
 
-        if (slave->priv->greeter != NULL) {
-                greeter_session = gdm_welcome_session_get_session (GDM_WELCOME_SESSION (slave->priv->greeter));
+        if (slave->priv->greeter_environment != NULL) {
+                greeter_session = gdm_launch_environment_get_session (GDM_LAUNCH_ENVIRONMENT (slave->priv->greeter_environment));
                 greeter_uid = gdm_session_get_allowed_user (greeter_session);
         } else {
                 greeter_uid = 0;
@@ -690,28 +800,28 @@ create_new_session (GdmSimpleSlave  *slave)
 }
 
 static void
-on_greeter_session_opened (GdmWelcomeSession *greeter,
-                           GdmSimpleSlave    *slave)
+on_greeter_environment_session_opened (GdmLaunchEnvironment *greeter_environment,
+                                       GdmSimpleSlave       *slave)
 {
         char       *session_id;
 
         g_debug ("GdmSimpleSlave: Greeter session opened");
-        session_id = gdm_welcome_session_get_session_id (GDM_WELCOME_SESSION (greeter));
+        session_id = gdm_launch_environment_get_session_id (GDM_LAUNCH_ENVIRONMENT (greeter_environment));
 
         g_object_set (GDM_SLAVE (slave), "session-id", session_id, NULL);
         g_free (session_id);
 }
 
 static void
-on_greeter_session_started (GdmWelcomeSession *greeter,
-                            GdmSimpleSlave    *slave)
+on_greeter_environment_session_started (GdmLaunchEnvironment *greeter_environment,
+                                        GdmSimpleSlave       *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter started");
 }
 
 static void
-on_greeter_session_stopped (GdmWelcomeSession *greeter,
-                            GdmSimpleSlave    *slave)
+on_greeter_environment_session_stopped (GdmLaunchEnvironment *greeter_environment,
+                                        GdmSimpleSlave       *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter stopped");
         if (slave->priv->start_session_service_name == NULL) {
@@ -720,15 +830,14 @@ on_greeter_session_stopped (GdmWelcomeSession *greeter,
                 start_session (slave);
         }
 
-        g_object_unref (slave->priv->greeter);
-        slave->priv->greeter = NULL;
-
+        g_object_unref (slave->priv->greeter_environment);
+        slave->priv->greeter_environment = NULL;
 }
 
 static void
-on_greeter_session_exited (GdmWelcomeSession    *greeter,
-                           int                   code,
-                           GdmSimpleSlave       *slave)
+on_greeter_environment_session_exited (GdmLaunchEnvironment    *greeter_environment,
+                                       int                      code,
+                                       GdmSimpleSlave          *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter exited: %d", code);
         if (slave->priv->start_session_service_name == NULL) {
@@ -737,9 +846,9 @@ on_greeter_session_exited (GdmWelcomeSession    *greeter,
 }
 
 static void
-on_greeter_session_died (GdmWelcomeSession    *greeter,
-                         int                   signal,
-                         GdmSimpleSlave       *slave)
+on_greeter_environment_session_died (GdmLaunchEnvironment    *greeter_environment,
+                                     int                      signal,
+                                     GdmSimpleSlave          *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter died: %d", signal);
         if (slave->priv->start_session_service_name == NULL) {
@@ -842,35 +951,60 @@ setup_server (GdmSimpleSlave *slave)
 #endif
 }
 
-static GdmWelcomeSession *
-create_greeter_session (const char *display_name,
-                        const char *seat_id,
-                        const char *display_device,
-                        const char *display_hostname,
-                        gboolean    display_is_local)
+static GdmLaunchEnvironment *
+create_environment (const char *session_id,
+                    const char *user_name,
+                    const char *display_name,
+                    const char *seat_id,
+                    const char *display_device,
+                    const char *display_hostname,
+                    gboolean    display_is_local)
 {
         gboolean debug = FALSE;
-        char *command = BINDIR "/gnome-session -f";
+        char *command;
+        GdmLaunchEnvironment *launch_environment;
+        char **argv;
+        GPtrArray *args;
 
         gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
 
+        args = g_ptr_array_new ();
+        g_ptr_array_add (args, BINDIR "/gnome-session -f");
+
         if (debug) {
-                command = BINDIR "/gnome-session -f --debug";
+                g_ptr_array_add (args, "--debug");
         }
 
-        return g_object_new (GDM_TYPE_WELCOME_SESSION,
-                             "command", command,
-                             "x11-display-name", display_name,
-                             "x11-display-seat-id", seat_id,
-                             "x11-display-device", display_device,
-                             "x11-display-hostname", display_hostname,
-                             "x11-display-is-local", display_is_local,
-                             "runtime-dir", GDM_SCREENSHOT_DIR,
-                             NULL);
+        if (session_id != NULL) {
+                g_ptr_array_add (args, " --session=");
+                g_ptr_array_add (args, (char *) session_id);
+        }
+
+        g_ptr_array_add (args, NULL);
+
+        argv = (char **) g_ptr_array_free (args, FALSE);
+        command = g_strjoinv (" ", argv);
+        g_free (argv);
+
+        launch_environment = g_object_new (GDM_TYPE_LAUNCH_ENVIRONMENT,
+                                           "command", command,
+                                           "user-name", user_name,
+                                           "x11-display-name", display_name,
+                                           "x11-display-seat-id", seat_id,
+                                           "x11-display-device", display_device,
+                                           "x11-display-hostname", display_hostname,
+                                           "x11-display-is-local", display_is_local,
+                                           "runtime-dir", GDM_SCREENSHOT_DIR,
+                                           NULL);
+
+        g_free (command);
+        return launch_environment;
 }
 
 static void
-start_greeter (GdmSimpleSlave *slave)
+start_launch_environment (GdmSimpleSlave *slave,
+                          char           *username,
+                          char           *session_id)
 {
         gboolean       display_is_local;
         char          *display_id;
@@ -921,36 +1055,38 @@ start_greeter (GdmSimpleSlave *slave)
         gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
 
         g_debug ("GdmSimpleSlave: Creating greeter on %s %s %s", display_name, display_device, display_hostname);
-        slave->priv->greeter = create_greeter_session (display_name,
-                                                       seat_id,
-                                                       display_device,
-                                                       display_hostname,
-                                                       display_is_local);
-        g_signal_connect (slave->priv->greeter,
+        slave->priv->greeter_environment = create_environment (session_id,
+                                                               GDM_USERNAME,
+                                                               display_name,
+                                                               seat_id,
+                                                               display_device,
+                                                               display_hostname,
+                                                               display_is_local);
+        g_signal_connect (slave->priv->greeter_environment,
                           "opened",
-                          G_CALLBACK (on_greeter_session_opened),
+                          G_CALLBACK (on_greeter_environment_session_opened),
                           slave);
-        g_signal_connect (slave->priv->greeter,
+        g_signal_connect (slave->priv->greeter_environment,
                           "started",
-                          G_CALLBACK (on_greeter_session_started),
+                          G_CALLBACK (on_greeter_environment_session_started),
                           slave);
-        g_signal_connect (slave->priv->greeter,
+        g_signal_connect (slave->priv->greeter_environment,
                           "stopped",
-                          G_CALLBACK (on_greeter_session_stopped),
+                          G_CALLBACK (on_greeter_environment_session_stopped),
                           slave);
-        g_signal_connect (slave->priv->greeter,
+        g_signal_connect (slave->priv->greeter_environment,
                           "exited",
-                          G_CALLBACK (on_greeter_session_exited),
+                          G_CALLBACK (on_greeter_environment_session_exited),
                           slave);
-        g_signal_connect (slave->priv->greeter,
+        g_signal_connect (slave->priv->greeter_environment,
                           "died",
-                          G_CALLBACK (on_greeter_session_died),
+                          G_CALLBACK (on_greeter_environment_session_died),
                           slave);
-        g_object_set (slave->priv->greeter,
+        g_object_set (slave->priv->greeter_environment,
                       "x11-authority-file", auth_file,
                       NULL);
 
-        gdm_welcome_session_start (GDM_WELCOME_SESSION (slave->priv->greeter));
+        gdm_launch_environment_start (GDM_LAUNCH_ENVIRONMENT (slave->priv->greeter_environment));
 
         g_free (display_id);
         g_free (display_name);
@@ -960,6 +1096,142 @@ start_greeter (GdmSimpleSlave *slave)
         g_free (auth_file);
 }
 
+static void
+start_greeter (GdmSimpleSlave *slave)
+{
+        start_launch_environment (slave, GDM_USERNAME, NULL);
+}
+
+#define RULES_DIR LOCALSTATEDIR "/lib/polkit-1/localauthority/10-vendor.d/"
+#define RULES_FILE "20-gnome-initial-setup.rules"
+
+static const gboolean
+create_initial_setup_user (GdmSimpleSlave *slave)
+{
+        gboolean ret = TRUE;
+        ActUserManager *act;
+        ActUser *user;
+        const char *username;
+        GFile *src_file, *dest_file;
+        GError *error = NULL;
+        const char *e = NULL;
+
+        username = "gnome-initial-setup";
+
+        /* First, create the user */
+        act = act_user_manager_get_default ();
+
+        user = act_user_manager_create_user (act, username, "", 0, &error);
+        if (user == NULL) {
+                if (g_dbus_error_is_remote_error (error)) {
+                        e = g_dbus_error_get_remote_error (error);
+		}
+
+                g_warning ("Creating user '%s' failed: %s / %s", username, e, error->message);
+
+                if (g_strcmp0 (e, "org.freedesktop.Accounts.Error.UserExists") != 0) {
+                        ret = FALSE;
+                        goto out;
+                }
+        } else {
+                g_object_unref (user);
+        }
+
+        /* Now, make sure the PolicyKit policy is in place */
+        src_file = g_file_new_for_path (DATADIR "/gnome-initial-setup" RULES_FILE);
+        dest_file = g_file_new_for_path (RULES_DIR RULES_FILE);
+
+        if (!g_file_copy (src_file,
+                          dest_file,
+                          G_FILE_COPY_OVERWRITE,
+                          NULL, NULL, NULL, &error)) {
+                g_warning ("Failed to copy '%s' to '%s': %s",
+                           g_file_get_path (src_file),
+                           g_file_get_path (dest_file),
+                           error->message);
+                ret = FALSE;
+                goto out_clear_files;
+        }
+
+ out_clear_files:
+        g_object_unref (src_file);
+        g_object_unref (dest_file);
+
+ out:
+        g_clear_pointer (&e, g_free);
+        g_clear_error (&error);
+        return ret;
+}
+
+static void
+destroy_initial_setup_user (GdmSimpleSlave *slave)
+{
+        ActUserManager *act;
+        ActUser *user;
+        const char *username;
+        const char *filename;
+        GError *error;
+
+        username = "gnome-initial-setup";
+        filename = RULES_DIR RULES_FILE;
+
+        if (g_remove (filename) < 0) {
+                g_warning ("Failed to remove '%s': %s", filename, g_strerror (errno));
+        }
+
+        act = act_user_manager_get_default ();
+
+        error = NULL;
+        user = act_user_manager_get_user (act, username);
+        if (!act_user_manager_delete_user (act, user, TRUE, &error)) {
+                g_warning ("Failed to create user '%s': %s", username, error->message);
+                g_error_free (error);
+        }
+
+        g_object_unref (user);
+}
+
+static void
+start_initial_setup (GdmSimpleSlave *slave)
+{
+        create_initial_setup_user (slave);
+        start_launch_environment (slave, INITIAL_SETUP_USERNAME, "gnome-initial-setup");
+        destroy_initial_setup_user (slave);
+}
+
+static gboolean
+wants_autologin (GdmSimpleSlave *slave)
+{
+        gboolean enabled = FALSE;
+        int delay = 0;
+        /* FIXME: handle wait-for-go */
+
+        gdm_slave_get_timed_login_details (GDM_SLAVE (slave), &enabled, NULL, &delay);
+        return enabled && delay > 0;
+}
+
+#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "lib/gdm/run-initial-setup"
+
+static gboolean
+wants_initial_setup (GdmSimpleSlave *slave)
+{
+        gboolean enabled;
+
+        if (!g_file_test (INITIAL_SETUP_TRIGGER_FILE, G_FILE_TEST_EXISTS)) {
+                return FALSE;
+        }
+
+        if (!gdm_settings_direct_get_boolean (GDM_KEY_INITIAL_SETUP_ENABLE, &enabled)) {
+                return FALSE;
+        }
+
+        if (!enabled) {
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
 static gboolean
 idle_connect_to_display (GdmSimpleSlave *slave)
 {
@@ -969,23 +1241,16 @@ idle_connect_to_display (GdmSimpleSlave *slave)
 
         res = gdm_slave_connect_to_x11_display (GDM_SLAVE (slave));
         if (res) {
-                gboolean enabled;
-                int      delay;
-
-                /* FIXME: handle wait-for-go */
-
                 setup_server (slave);
 
-                delay = 0;
-                enabled = FALSE;
-                gdm_slave_get_timed_login_details (GDM_SLAVE (slave), &enabled, NULL, &delay);
-                if (! enabled || delay > 0) {
-                        start_greeter (slave);
-                } else {
+                if (wants_initial_setup (slave)) {
+                        start_initial_setup (slave);
+                } else if (wants_autologin (slave)) {
                         /* Run the init script. gdmslave suspends until script has terminated */
                         gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
+                } else {
+                        start_greeter (slave);
                 }
-
                 create_new_session (slave);
         } else {
                 if (slave->priv->connection_attempts >= MAX_CONNECT_ATTEMPTS) {
@@ -1046,12 +1311,14 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
         char    *auth_file;
         char    *seat_id;
         gboolean display_is_local;
+        gboolean display_is_initial;
 
         g_object_get (slave,
                       "display-is-local", &display_is_local,
                       "display-name", &display_name,
                       "display-seat-id", &seat_id,
                       "display-x11-authority-file", &auth_file,
+                      "display-is-initial", &display_is_initial,
                       NULL);
 
         /* if this is local display start a server if one doesn't
@@ -1060,7 +1327,7 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
                 gboolean res;
                 gboolean disable_tcp;
 
-                slave->priv->server = gdm_server_new (display_name, seat_id, auth_file);
+                slave->priv->server = gdm_server_new (display_name, seat_id, auth_file, display_is_initial);
 
                 disable_tcp = TRUE;
                 if (gdm_settings_client_get_boolean (GDM_KEY_DISALLOW_TCP,
@@ -1088,12 +1355,9 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
 
                 if (slave->priv->plymouth_is_running) {
                         plymouth_prepare_for_transition (slave);
-                        res = gdm_server_start_on_active_vt (slave->priv->server);
-                } else
-#endif
-                {
-                        res = gdm_server_start (slave->priv->server);
                 }
+#endif
+                res = gdm_server_start (slave->priv->server);
                 if (! res) {
                         g_warning (_("Could not start the X "
                                      "server (your graphical environment) "
@@ -1234,7 +1498,7 @@ gdm_simple_slave_stop (GdmSlave *slave)
 
         GDM_SLAVE_CLASS (gdm_simple_slave_parent_class)->stop (slave);
 
-        if (self->priv->greeter != NULL) {
+        if (self->priv->greeter_environment != NULL) {
                 stop_greeter (self);
         }
 
@@ -1254,6 +1518,9 @@ gdm_simple_slave_stop (GdmSlave *slave)
                 gdm_simple_slave_revoke_console_permissions (self);
 #endif
 
+        }
+
+        if (self->priv->session != NULL) {
                 gdm_session_close (self->priv->session);
                 g_clear_object (&self->priv->session);
         }
@@ -1266,55 +1533,12 @@ gdm_simple_slave_stop (GdmSlave *slave)
         return TRUE;
 }
 
-static void
-gdm_simple_slave_set_property (GObject      *object,
-                               guint          prop_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-gdm_simple_slave_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue      *value,
-                               GParamSpec *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static GObject *
-gdm_simple_slave_constructor (GType                  type,
-                              guint                  n_construct_properties,
-                              GObjectConstructParam *construct_properties)
-{
-        GdmSimpleSlave      *simple_slave;
-
-        simple_slave = GDM_SIMPLE_SLAVE (G_OBJECT_CLASS (gdm_simple_slave_parent_class)->constructor (type,
-                                                                                 n_construct_properties,
-                                                                                 construct_properties));
-
-        return G_OBJECT (simple_slave);
-}
-
 static void
 gdm_simple_slave_class_init (GdmSimpleSlaveClass *klass)
 {
         GObjectClass  *object_class = G_OBJECT_CLASS (klass);
         GdmSlaveClass *slave_class = GDM_SLAVE_CLASS (klass);
 
-        object_class->get_property = gdm_simple_slave_get_property;
-        object_class->set_property = gdm_simple_slave_set_property;
-        object_class->constructor = gdm_simple_slave_constructor;
         object_class->finalize = gdm_simple_slave_finalize;
 
         slave_class->start = gdm_simple_slave_start;
