@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -43,6 +43,7 @@
 #include "gdm-manager.h"
 #include "gdm-log.h"
 #include "gdm-common.h"
+#include "gdm-signal-handler.h"
 
 #include "gdm-settings.h"
 #include "gdm-settings-direct.h"
@@ -144,54 +145,128 @@ write_pid (void)
         atexit (delete_pid);
 }
 
-static gboolean
-ensure_dir_with_perms (const char *path,
-                       uid_t       uid,
-                       gid_t       gid,
-                       mode_t      mode,
-                       GError    **error)
-{
-        gboolean ret = FALSE;
-
-        if (g_mkdir_with_parents (path, 0755) == -1) {
-                g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno), g_strerror (errno));
-                goto out;
-        }
-        if (g_chmod (path, mode) == -1) {
-                g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno), g_strerror (errno));
-                goto out;
-        }
-        if (chown (path, uid, gid) == -1) {
-                g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno), g_strerror (errno));
-                goto out;
-        }
-
-        ret = TRUE;
- out:
-        return ret;
-}
-
 static void
-gdm_daemon_ensure_dirs (uid_t uid,
-                        gid_t gid)
+check_logdir (void)
 {
-        GError *error = NULL;
+        struct stat     statbuf;
+        int             r;
+        const char     *log_path;
 
-        /* Set up /var/run/gdm */
-        if (!ensure_dir_with_perms (GDM_RAN_ONCE_MARKER_DIR, 0, gid, 0711, &error)) {
-                gdm_fail (_("Failed to create ran once marker dir %s: %s"),
-                          GDM_RAN_ONCE_MARKER_DIR, error->message);
-        }
+        log_path = LOGDIR;
 
-        /* Set up /var/log/gdm */
-        if (!ensure_dir_with_perms (LOGDIR, 0, gid, 0711, &error)) {
-                gdm_fail (_("Failed to create LogDir %s: %s"),
-                          LOGDIR, error->message);
+        r = g_stat (log_path, &statbuf);
+        if (r < 0 || ! S_ISDIR (statbuf.st_mode))  {
+                if (g_mkdir (log_path, 0755) < 0) {
+                        gdm_fail (_("Logdir %s does not exist or isn't a directory."),
+                                  log_path);
+                }
+                g_chmod (log_path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
         }
 }
 
 static void
-gdm_daemon_lookup_user (uid_t *uidp,
+check_servauthdir (const char  *auth_path,
+                   struct stat *statbuf)
+{
+        int r;
+
+        /* Enter paranoia mode */
+        r = g_stat (auth_path, statbuf);
+        if (r < 0) {
+                gdm_fail (_("Authdir %s does not exist. Aborting."), auth_path);
+        }
+
+        if (! S_ISDIR (statbuf->st_mode)) {
+                gdm_fail (_("Authdir %s is not a directory. Aborting."), auth_path);
+        }
+}
+
+static void
+set_effective_user (uid_t uid)
+{
+        int res;
+
+        res = 0;
+
+        if (geteuid () != uid) {
+                res = seteuid (uid);
+        }
+
+        if (res != 0) {
+                g_error ("Cannot set uid to %d: %s",
+                         (int)uid,
+                         g_strerror (errno));
+        }
+}
+
+static void
+set_effective_group (gid_t gid)
+{
+        int res;
+
+        res = 0;
+        if (getegid () != gid) {
+                res = setegid (gid);
+        }
+
+        if (res != 0) {
+                g_error ("Cannot set gid to %d: %s",
+                         (int)gid,
+                         g_strerror (errno));
+        }
+}
+
+static void
+set_effective_user_group (uid_t uid,
+                          gid_t gid)
+{
+        set_effective_user (0);
+        set_effective_group (gid);
+        if (uid != 0) {
+                set_effective_user (0);
+        }
+}
+
+static void
+gdm_daemon_check_permissions (uid_t uid,
+                              gid_t gid)
+{
+        struct stat statbuf;
+        const char *auth_path;
+
+        auth_path = LOGDIR;
+
+        /* Enter paranoia mode */
+        check_servauthdir (auth_path, &statbuf);
+
+        set_effective_user_group (0, 0);
+
+        /* Now set things up for us as  */
+        chown (auth_path, 0, gid);
+        g_chmod (auth_path, (S_IRWXU|S_IRWXG|S_ISVTX));
+
+        set_effective_user_group (uid, gid);
+
+        /* Again paranoid */
+        check_servauthdir (auth_path, &statbuf);
+
+        if G_UNLIKELY (statbuf.st_uid != 0 || statbuf.st_gid != gid)  {
+                gdm_fail (_("Authdir %s is not owned by user %d, group %d. Aborting."),
+                          auth_path,
+                          (int)uid,
+                          (int)gid);
+        }
+
+        if G_UNLIKELY (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
+                gdm_fail (_("Authdir %s has wrong permissions %o. Should be %o. Aborting."),
+                          auth_path,
+                          statbuf.st_mode,
+                          (S_IRWXU|S_IRWXG|S_ISVTX));
+        }
+}
+
+static void
+gdm_daemon_change_user (uid_t *uidp,
                         gid_t *gidp)
 {
         char          *username;
@@ -220,7 +295,7 @@ gdm_daemon_lookup_user (uid_t *uidp,
 
         /* Set uid and gid */
         if G_UNLIKELY (pwent == NULL) {
-                gdm_fail (_("Can’t find the GDM user “%s”. Aborting!"), username);
+                gdm_fail (_("Can't find the GDM user '%s'. Aborting!"), username);
         } else {
                 uid = pwent->pw_uid;
         }
@@ -232,7 +307,7 @@ gdm_daemon_lookup_user (uid_t *uidp,
         grent = getgrnam (groupname);
 
         if G_UNLIKELY (grent == NULL) {
-                gdm_fail (_("Can’t find the GDM group “%s”. Aborting!"), groupname);
+                gdm_fail (_("Can't find the GDM group '%s'. Aborting!"), groupname);
         } else  {
                 gid = grent->gr_gid;
         }
@@ -241,6 +316,9 @@ gdm_daemon_lookup_user (uid_t *uidp,
                 gdm_fail (_("The GDM group should not be root. Aborting!"));
         }
 
+        /* gid remains 'gdm' */
+        set_effective_user_group (uid, gid);
+
         if (uidp != NULL) {
                 *uidp = uid;
         }
@@ -254,62 +332,94 @@ gdm_daemon_lookup_user (uid_t *uidp,
 }
 
 static gboolean
-on_shutdown_signal_cb (gpointer user_data)
+signal_cb (int      signo,
+           gpointer data)
 {
-        GMainLoop *mainloop = user_data;
+        int ret;
 
-        g_main_loop_quit (mainloop);
+        g_debug ("Got callback for signal %d", signo);
 
-        return FALSE;
-}
+        ret = TRUE;
 
-static gboolean
-on_sighup_cb (gpointer user_data)
-{
-        g_debug ("Got HUP signal");
-        /* Reread config stuff like system config files, VPN service
-         * files, etc
-         */
-        g_object_unref (settings);
-        settings = gdm_settings_new ();
-        if (settings != NULL) {
-                if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
-                        g_warning ("Unable to initialize settings");
+        switch (signo) {
+        case SIGFPE:
+        case SIGPIPE:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down abnormally.", signo);
+                ret = FALSE;
+
+                break;
+
+        case SIGINT:
+        case SIGTERM:
+                /* let the fatal signals interrupt us */
+                g_debug ("Caught signal %d, shutting down normally.", signo);
+                ret = FALSE;
+
+                break;
+
+        case SIGHUP:
+                g_debug ("Got HUP signal");
+                /* Reread config stuff like system config files, VPN service
+                 * files, etc
+                 */
+                g_object_unref (settings);
+                settings = gdm_settings_new ();
+                if (settings != NULL) {
+                        if (! gdm_settings_direct_init (settings, GDMCONFDIR "/gdm.schemas", "/")) {
+                                g_warning ("Unable to initialize settings");
+                        }
                 }
+
+                ret = TRUE;
+
+                break;
+
+        case SIGUSR1:
+                g_debug ("Got USR1 signal");
+                /* FIXME:
+                 * Play with log levels or something
+                 */
+                ret = TRUE;
+
+                gdm_log_toggle_debug ();
+
+                break;
+
+        default:
+                g_debug ("Caught unhandled signal %d", signo);
+                ret = TRUE;
+
+                break;
         }
 
-        return TRUE;
+        return ret;
 }
 
 static gboolean
 is_debug_set (void)
 {
-        gboolean debug;
+        gboolean debug = FALSE;
+
+        /* enable debugging for unstable builds */
+        if (gdm_is_version_unstable ()) {
+                return TRUE;
+        }
+
         gdm_settings_direct_get_boolean (GDM_KEY_DEBUG, &debug);
         return debug;
 }
 
-/* SIGUSR1 is used by the X server to tell us that we're ready, so
- * block it. We'll unblock it in the worker thread in gdm-server.c
- */
-static void
-block_sigusr1 (void)
-{
-        sigset_t mask;
-
-        sigemptyset (&mask);
-        sigaddset (&mask, SIGUSR1);
-        sigprocmask (SIG_BLOCK, &mask, NULL);
-}
-
 int
 main (int    argc,
       char **argv)
 {
         GMainLoop          *main_loop;
         GOptionContext     *context;
-        GError             *error = NULL;
+        GError             *error;
+        int                 ret;
         gboolean            res;
+        GdmSignalHandler   *signal_handler;
         static gboolean     do_timed_exit    = FALSE;
         static gboolean     print_version    = FALSE;
         static gboolean     fatal_warnings   = FALSE;
@@ -321,34 +431,30 @@ main (int    argc,
                 { NULL }
         };
 
-        block_sigusr1 ();
-
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
         textdomain (GETTEXT_PACKAGE);
         setlocale (LC_ALL, "");
 
+        ret = 1;
+
+        g_type_init ();
+
         context = g_option_context_new (_("GNOME Display Manager"));
         g_option_context_add_main_entries (context, entries, NULL);
+        g_option_context_set_ignore_unknown_options (context, TRUE);
 
         error = NULL;
         res = g_option_context_parse (context, &argc, &argv, &error);
         g_option_context_free (context);
         if (! res) {
-                g_printerr ("Failed to parse options: %s\n", error->message);
+                g_warning ("%s", error->message);
                 g_error_free (error);
-                return EXIT_FAILURE;
+                goto out;
         }
 
         if (print_version) {
                 g_print ("GDM %s\n", VERSION);
-                return EXIT_SUCCESS;
-        }
-
-        /* XDM compliant error message */
-        if (getuid () != 0) {
-                /* make sure the pid file doesn't get wiped */
-                g_printerr ("%s\n", _("Only the root user can run GDM"));
-                return EXIT_FAILURE;
+                exit (1);
         }
 
         if (fatal_warnings) {
@@ -362,16 +468,30 @@ main (int    argc,
         gdm_log_init ();
 
         settings = gdm_settings_new ();
+        if (settings == NULL) {
+                g_warning ("Unable to initialize settings");
+                goto out;
+        }
+
         if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
                 g_warning ("Unable to initialize settings");
-                return EXIT_FAILURE;
+                goto out;
         }
 
         gdm_log_set_debug (is_debug_set ());
 
-        gdm_daemon_lookup_user (&gdm_uid, &gdm_gid);
+        gdm_daemon_change_user (&gdm_uid, &gdm_gid);
+        gdm_daemon_check_permissions (gdm_uid, gdm_gid);
 
-        gdm_daemon_ensure_dirs (gdm_uid, gdm_gid);
+        set_effective_user_group (0, 0);
+        check_logdir ();
+
+        /* XDM compliant error message */
+        if (getuid () != 0) {
+                /* make sure the pid file doesn't get wiped */
+                g_warning (_("Only the root user can run GDM"));
+                exit (-1);
+        }
 
         /* Connect to the bus, own the name and start the manager */
         bus_reconnect ();
@@ -380,13 +500,20 @@ main (int    argc,
         delete_pid ();
         write_pid ();
 
-        g_chdir ("/");
+        g_chdir (AUTHDIR);
 
         main_loop = g_main_loop_new (NULL, FALSE);
 
-        g_unix_signal_add (SIGTERM, on_shutdown_signal_cb, main_loop);
-        g_unix_signal_add (SIGINT, on_shutdown_signal_cb, main_loop);
-        g_unix_signal_add (SIGHUP, on_sighup_cb, NULL);
+        signal_handler = gdm_signal_handler_new ();
+        gdm_signal_handler_set_fatal_func (signal_handler,
+                                           (GDestroyNotify)g_main_loop_quit,
+                                           main_loop);
+        gdm_signal_handler_add_fatal (signal_handler);
+        gdm_signal_handler_add (signal_handler, SIGTERM, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGINT, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGFPE, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
+        gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
 
         if (do_timed_exit) {
                 g_timeout_add_seconds (30, (GSourceFunc) timed_exit_cb, main_loop);
@@ -398,13 +525,18 @@ main (int    argc,
 
         g_clear_object (&manager);
         g_clear_object (&settings);
+        g_clear_object (&signal_handler);
 
         gdm_settings_direct_shutdown ();
         gdm_log_shutdown ();
 
         g_main_loop_unref (main_loop);
 
-        return EXIT_SUCCESS;
+        ret = 0;
+
+ out:
+
+        return ret;
 }
 
 static void
@@ -413,25 +545,20 @@ on_name_acquired (GDBusConnection *bus,
                   gpointer         user_data)
 {
         gboolean xdmcp_enabled;
-        gboolean show_local_greeter;
 
         manager = gdm_manager_new ();
         if (manager == NULL) {
                 g_warning ("Could not construct manager object");
-                exit (EXIT_FAILURE);
+                exit (1);
         }
 
         g_debug ("Successfully connected to D-Bus");
 
-        show_local_greeter = TRUE;
-        gdm_settings_direct_get_boolean (GDM_KEY_SHOW_LOCAL_GREETER, &show_local_greeter);
-        gdm_manager_set_show_local_greeter (manager, show_local_greeter);
+        gdm_manager_start (manager);
 
         xdmcp_enabled = FALSE;
         gdm_settings_direct_get_boolean (GDM_KEY_XDMCP_ENABLE, &xdmcp_enabled);
         gdm_manager_set_xdmcp_enabled (manager, xdmcp_enabled);
-
-        gdm_manager_start (manager);
 }
 
 static void
